/*
 * Copyright (C) 1994-2018 Altair Engineering, Inc.
 * For more information, contact Altair at www.altair.com.
 *
 * This file is part of the PBS Professional ("PBS Pro") software.
 *
 * Open Source License Information:
 *
 * PBS Pro is free software. You can redistribute it and/or modify it under the
 * terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * PBS Pro is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Commercial License Information:
 *
 * For a copy of the commercial license terms and conditions,
 * go to: (http://www.pbspro.com/UserArea/agreement.html)
 * or contact the Altair Legal Department.
 *
 * Altair’s dual-license business model allows companies, individuals, and
 * organizations to create proprietary derivative works of PBS Pro and
 * distribute them - whether embedded or bundled with other software -
 * under a commercial license agreement.
 *
 * Use of Altair’s trademarks, including but not limited to "PBS™",
 * "PBS Professional®", and "PBS Pro™" and Altair’s logos is subject to Altair's
 * trademark licensing policies.
 *
 */


/**
 * @file    db_postgres_sched.c
 *
 * @brief
 *      Implementation of the scheduler data access functions for postgres
 */
#include <pbs_config.h>   /* the master config generated by configure */
#include "pbs_db.h"
#include "db_postgres.h"

/**
 * @brief
 *	Prepare all the scheduler related sqls. Typically called after connect
 *	and before any other sql exeuction
 *
 * @param[in]	conn - Database connection handle
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 *
 */
int
pg_db_prepare_sched_sqls(pbs_db_conn_t *conn)
{
	sprintf(conn->conn_sql, "insert into "
		"pbs.scheduler( "
		"sched_name, "
		"sched_sv_name, "
		"sched_savetm, "
		"sched_creattm "
		") "
		"values ($1, $2, localtimestamp, localtimestamp)");
	if (pg_prepare_stmt(conn, STMT_INSERT_SCHED, conn->conn_sql, 2) != 0)
		return -1;

	sprintf(conn->conn_sql, "update pbs.scheduler set "
		"sched_sv_name = $2, "
		"sched_savetm = localtimestamp "
		"where sched_name = $1");
	if (pg_prepare_stmt(conn, STMT_UPDATE_SCHED, conn->conn_sql, 2) != 0)
		return -1;

	sprintf(conn->conn_sql, "select "
		"sched_name, "
		"sched_sv_name, "
		"extract(epoch from sched_savetm)::bigint as sched_savetm, "
		"extract(epoch from sched_creattm)::bigint as sched_creattm "
		"from "
		"pbs.scheduler "
		"where sched_name = $1");
	if (pg_prepare_stmt(conn, STMT_SELECT_SCHED, conn->conn_sql, 1) != 0)
		return -1;

	sprintf(conn->conn_sql, "select "
		"sched_name, "
		"sched_sv_name, "
		"extract(epoch from sched_savetm)::bigint as sched_savetm, "
		"extract(epoch from sched_creattm)::bigint as sched_creattm "
		"from "
		"pbs.scheduler ");
	if (pg_prepare_stmt(conn, STMT_SELECT_SCHED_ALL, conn->conn_sql, 1) != 0)
		return -1;

	sprintf(conn->conn_sql, "insert into "
		"pbs.scheduler_attr "
		"(sched_name, "
		"attr_name, "
		"attr_resource, "
		"attr_value, "
		"attr_flags) "
		"values "
		"($1, $2, $3, $4, $5)");
	if (pg_prepare_stmt(conn, STMT_INSERT_SCHEDATTR, conn->conn_sql, 5) != 0)
		return -1;

	sprintf(conn->conn_sql, "update pbs.scheduler_attr set "
		"attr_resource = $3, "
		"attr_value = $4, "
		"attr_flags = $5 "
		"where sched_name = $1 "
		"and attr_name = $2");
	if (pg_prepare_stmt(conn, STMT_UPDATE_SCHEDATTR, conn->conn_sql, 5) != 0)
		return -1;

	sprintf(conn->conn_sql, "update pbs.scheduler_attr set "
		"attr_value = $4, "
		"attr_flags = $5 "
		"where sched_name = $1 "
		"and attr_name = $2 "
		"and attr_resource = $3");
	if (pg_prepare_stmt(conn, STMT_UPDATE_SCHEDATTR_RESC,
		conn->conn_sql, 5) != 0)
		return -1;

	sprintf(conn->conn_sql, "delete from "
		"pbs.scheduler_attr "
		"where sched_name = $1 "
		"and attr_name = $2");
	if (pg_prepare_stmt(conn, STMT_DELETE_SCHEDATTR, conn->conn_sql, 2) != 0)
		return -1;

	sprintf(conn->conn_sql, "delete from "
		"pbs.scheduler_attr "
		"where sched_name = $1 "
		"and attr_name = $2 "
		"and attr_resource = $3");
	if (pg_prepare_stmt(conn, STMT_DELETE_SCHEDATTR_RESC,
		conn->conn_sql, 3) != 0)
		return -1;

	sprintf(conn->conn_sql, "select "
		"attr_name, "
		"attr_resource, "
		"attr_value, "
		"attr_flags "
		"from "
		"pbs.scheduler_attr "
		"where sched_name = $1");
	if (pg_prepare_stmt(conn, STMT_SELECT_SCHEDATTR, conn->conn_sql, 1) != 0)
		return -1;

	sprintf(conn->conn_sql, "delete from pbs.scheduler where sched_name = $1");
	if (pg_prepare_stmt(conn, STMT_DELETE_SCHED, conn->conn_sql, 1) != 0)
		return -1;

	return 0;
}

/**
 * @brief
 *	Insert scheduler data into the database
 *
 * @param[in]	conn - Connection handle
 * @param[in]	obj  - Information of scheduler to be inserted
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 *
 */
int
pg_db_insert_sched(pbs_db_conn_t *conn, pbs_db_obj_info_t *obj)
{
	pbs_db_sched_info_t *psch = obj->pbs_db_un.pbs_db_sched;

	LOAD_STR(conn, psch->sched_name, 0);
	LOAD_STR(conn, psch->sched_sv_name, 1);

	if (pg_db_cmd(conn, STMT_INSERT_SCHED, 2) != 0)
		return -1;

	return 0;
}

/**
 * @brief
 *	Update scheduler data into the database
 *
 * @param[in]	conn - Connection handle
 * @param[in]	obj  - Information of scheduler to be updated
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 * @retval	 1 - Success but no rows updated
 */
int
pg_db_update_sched(pbs_db_conn_t *conn, pbs_db_obj_info_t *obj)
{
	pbs_db_sched_info_t *psch = obj->pbs_db_un.pbs_db_sched;

	LOAD_STR(conn, psch->sched_name, 0);
	LOAD_STR(conn, psch->sched_sv_name, 1);

	return (pg_db_cmd(conn, STMT_UPDATE_SCHED, 2));
}

/**
 * @brief
 *	Load scheduler data from the row into the scheduler object
 *
 * @param[in]	res - Resultset from a earlier query
 * @param[out]	psch  - Scheduler object to load data into
 * @param[in]	row - The current row to load within the resultset
 *
 */
static void
load_sched(PGresult *res, pbs_db_sched_info_t *psch, int row)
{

	strncpy(psch->sched_name, PQgetvalue(res, row,
		PQfnumber(res, "sched_name")), PBS_MAXSCHEDNAME);
	psch->sched_name[PBS_MAXSCHEDNAME] = '\0';
	strncpy(psch->sched_sv_name, PQgetvalue(res, row,
		PQfnumber(res, "sched_sv_name")), PBS_MAXSCHEDNAME);
	psch->sched_sv_name[PBS_MAXSCHEDNAME] = '\0';
	psch->sched_savetm = strtoll(PQgetvalue(res, row,
		PQfnumber(res, "sched_savetm")), NULL, 10);
	psch->sched_creattm = strtoll(PQgetvalue(res, row,
		PQfnumber(res, "sched_creattm")), NULL, 10);

}

/**
 * @brief
 *	Load scheduler data from the database
 *
 * @param[in]	conn - Connection handle
 * @param[out]	obj  - Load scheduler information into this object
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 * @retval	 1 -  Success but no rows loaded
 *
 */
int
pg_db_load_sched(pbs_db_conn_t *conn, pbs_db_obj_info_t *obj)
{
	PGresult *res;
	int rc;
	pbs_db_sched_info_t *psch = obj->pbs_db_un.pbs_db_sched;

	LOAD_STR(conn, psch->sched_name, 0);
	if ((rc = pg_db_query(conn, STMT_SELECT_SCHED,  1, &res)) != 0)
		return rc;

	load_sched(res, psch, 0);

	PQclear(res);
	return 0;
}

/**
 * @brief
 *	Find scheduler
 *
 * @param[in]	conn - Connection handle
 * @param[out]	st   - The cursor state variable updated by this query
 * @param[in]	obj  - Information of sched to be found
 * @param[in]	opts - Any other options (like flags, timestamp)
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 * @retval	 1 -  Success but no rows found
 *
 */
int
pg_db_find_sched(pbs_db_conn_t *conn, void *st, pbs_db_obj_info_t *obj,
	pbs_db_query_options_t *opts)
{
	PGresult *res;
	pg_query_state_t *state = (pg_query_state_t *) st;
	int rc;
	int params;

	if (!state)
		return -1;
	strncpy(conn->conn_sql, STMT_SELECT_SCHED_ALL, (MAX_SQL_LENGTH-1));
	conn->conn_sql[MAX_SQL_LENGTH-1] = '\0';
	params = 0;

	if ((rc = pg_db_query(conn, conn->conn_sql, params, &res)) != 0)
		return rc;

	state->row = 0;
	state->res = res;
	state->count = PQntuples(res);

	return 0;
}

/**
 * @brief
 *	Get the next scheduler from the cursor
 *
 * @param[in]	conn - Connection handle
 * @param[out]	st   - The cursor state
 * @param[in]	obj  - Scheduler information is loaded into this object
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 *
 */
int
pg_db_next_sched(pbs_db_conn_t *conn, void *st, pbs_db_obj_info_t *obj)
{
	pg_query_state_t *state = (pg_query_state_t *) st;

	load_sched(state->res, obj->pbs_db_un.pbs_db_sched, state->row);

	return 0;
}

/**
 * @brief
 *	Delete the scheduler from the database
 *
 * @param[in]	conn - Connection handle
 * @param[in]	obj  - scheduler information
 *
 * @return      Error code
 * @retval	-1 - Failure
 * @retval	 0 - Success
 * @retval	 1 -  Success but no rows deleted
 *
 */
int
pg_db_delete_sched(pbs_db_conn_t *conn, pbs_db_obj_info_t *obj)
{
	pbs_db_sched_info_t *sc = obj->pbs_db_un.pbs_db_sched;
	LOAD_STR(conn, sc->sched_name, 0);
	return (pg_db_cmd(conn, STMT_DELETE_SCHED, 1));
}
